import'./ChangesHighlighter.js';import'./ChangesView.js';import'./ChangesSidebar.js';import*as ChangesHighlighter from'./ChangesHighlighter.js';import*as ChangesSidebar from'./ChangesSidebar.js';import*as ChangesView from'./ChangesView.js';export{ChangesHighlighter,ChangesSidebar,ChangesView,};