import'./clike.js';import'./coffeescript.js';import'./php.js';import'./python.js';import'./shell.js';import'./livescript.js';import'./markdown.js';import'./clojure.js';import'./jsx.js';import*as DefaultCodeMirrorMimeMode from'./DefaultCodeMirrorMimeMode.js';export{DefaultCodeMirrorMimeMode,};