SecurityTestRunner.dumpSecurityPanelSidebarOrigins=function(){for(const key in Security.SecurityPanelSidebarTree.OriginGroup){const originGroup=Security.SecurityPanelSidebarTree.OriginGroup[key];const element=Security.SecurityPanel._instance()._sidebarTree._originGroups.get(originGroup);if(element.hidden){continue;}
TestRunner.addResult('Group: '+element.title);const originTitles=element.childrenListElement.getElementsByTagName('span');for(const originTitle of originTitles){if(originTitle.className!=='tree-element-title'){TestRunner.dumpDeepInnerHTML(originTitle);}}}};SecurityTestRunner.dispatchRequestFinished=function(request){TestRunner.networkManager.dispatchEventToListeners(SDK.NetworkManager.Events.RequestFinished,request);};;