import'./InspectorFrontendHostAPI.js';import'./InspectorFrontendHost.js';import'./ResourceLoader.js';import'./UserMetrics.js';import'./Platform.js';import*as InspectorFrontendHost from'./InspectorFrontendHost.js';import*as Platform from'./Platform.js';import*as ResourceLoader from'./ResourceLoader.js';import*as UserMetrics from'./UserMetrics.js';export{InspectorFrontendHost,Platform,ResourceLoader,UserMetrics,};