export const PortPadding=4;export const InputPortRadius=10;export const AudioParamRadius=5;export const LeftMarginOfText=12;export const RightMarginOfText=30;export const LeftSideTopPadding=5;export const BottomPaddingWithoutParam=6;export const BottomPaddingWithParam=8;export const ArrowHeadSize=12;export const GraphPadding=20;export const GraphMargin=20;export const TotalInputPortHeight=InputPortRadius*2+PortPadding;export const TotalOutputPortHeight=TotalInputPortHeight;export const TotalParamPortHeight=AudioParamRadius*2+PortPadding;export const NodeLabelFontStyle='14px Segoe UI, Arial';export const ParamLabelFontStyle='12px Segoe UI, Arial';export const GraphStyles={PortPadding,InputPortRadius,AudioParamRadius,LeftMarginOfText,RightMarginOfText,LeftSideTopPadding,BottomPaddingWithoutParam,BottomPaddingWithParam,ArrowHeadSize,GraphPadding,GraphMargin,TotalInputPortHeight,TotalOutputPortHeight,TotalParamPortHeight,NodeLabelFontStyle,ParamLabelFontStyle};self.WebAudio=self.WebAudio||{};WebAudio=WebAudio||{};WebAudio.GraphVisualizer=WebAudio.GraphVisualizer||{};WebAudio.GraphVisualizer.GraphStyles=GraphStyles;