import'./Runtime.js';import'./platform/platform.js';import'./dom_extension/dom_extension.js';import'./common/common-legacy.js';import'./host/host.js';import'./protocol/protocol.js';import'./sdk/sdk.js';import'./ui/ui.js';import'./services/services.js';import'./workspace/workspace.js';import'./bindings/bindings.js';import'./components/components.js';import'./persistence/persistence.js';import'./browser_sdk/browser_sdk.js';import'./extensions/extensions.js';import'./console_counters/console_counters.js';import'./text_utils/text_utils.js';