import'./AllocationProfile.js';import'./HeapSnapshot.js';import'./HeapSnapshotLoader.js';import'./HeapSnapshotWorkerDispatcher.js';import'./HeapSnapshotWorker.js';import*as AllocationProfile from'./AllocationProfile.js';import*as HeapSnapshot from'./HeapSnapshot.js';import*as HeapSnapshotLoader from'./HeapSnapshotLoader.js';import*as HeapSnapshotWorkerDispatcher from'./HeapSnapshotWorkerDispatcher.js';export{AllocationProfile,HeapSnapshot,HeapSnapshotLoader,HeapSnapshotWorkerDispatcher,};