import'./PlatformFileSystem.js';import*as Automapping from'./Automapping.js';import*as EditFileSystemView from'./EditFileSystemView.js';import*as FileSystemWorkspaceBinding from'./FileSystemWorkspaceBinding.js';import*as IsolatedFileSystem from'./IsolatedFileSystem.js';import*as IsolatedFileSystemManager from'./IsolatedFileSystemManager.js';import*as NetworkPersistenceManager from'./NetworkPersistenceManager.js';import*as PersistenceActions from'./PersistenceActions.js';import*as Persistence from'./PersistenceImpl.js';import*as PersistenceUtils from'./PersistenceUtils.js';import*as PlatformFileSystem from'./PlatformFileSystem.js';import*as WorkspaceSettingsTab from'./WorkspaceSettingsTab.js';export{Automapping,EditFileSystemView,FileSystemWorkspaceBinding,IsolatedFileSystem,IsolatedFileSystemManager,NetworkPersistenceManager,Persistence,PersistenceActions,PersistenceUtils,PlatformFileSystem,WorkspaceSettingsTab,};