Root.allDescriptors.push(...[{"extensions":[{"className":"JsMain.JsMain","type":"early-initialization"}],"dependencies":["components"],"modules":["js_main.js","JsMain.js"],"name":"js_main","scripts":[]},{"dependencies":["profiler"],"extensions":[{"title":"Profiler","id":"js_profiler","className":"Profiler.JSProfilerPanel","location":"panel","type":"view","order":65}],"name":"js_profiler"}]);Root.applicationDescriptor.modules.push(...[{"type":"autostart","name":"js_main"},{"name":"js_profiler"}]);self['JsMain']=self['JsMain']||{};;