export class JavaScriptMetaData{signaturesForNativeFunction(name){}
signaturesForInstanceMethod(name,receiverClassName){}
signaturesForStaticMethod(name,receiverConstructorName){}}
self.Common=self.Common||{};Common=Common||{};Common.JavaScriptMetadata=JavaScriptMetaData;