Root.Runtime.cachedResources["protocol_monitor/protocolMonitor.css"]=".data-grid {\n    border: none;\n}\n.data-grid {\n    flex: auto;\n    border: none;\n}\n\n.data-grid .data {\n    background-image: none;\n}\n\n.data-grid td {\n    /* border-bottom: 1px solid #ccc; */\n    border-left-color: #ccc;\n}\n\n.data-grid tr.selected {\n    background-color: #def;\n}\n\n.data-grid th {\n    border-left-color: #ccc;\n}\n\n.protocol-message-sent {\n    background-color: hsl(281, 64%, 95%);\n}\n\n.protocol-monitor-toolbar {\n    border-bottom:var(--divider-border);\n}\n\n.error {\n    background-color: hsl(0, 100%, 97%);\n    color: red;\n}\n/*# sourceURL=protocol_monitor/protocolMonitor.css */";