Root.Runtime.cachedResources["event_listeners/eventListenersView.css"]="/*\n * Copyright 2015 The Chromium Authors. All rights reserved.\n * Use of this source code is governed by a BSD-style license that can be\n * found in the LICENSE file.\n */\n\n.tree-outline-disclosure li {\n    padding: 2px 0 0 5px;\n    overflow: hidden;\n    display: flex;\n    min-height: 17px;\n    align-items: baseline;\n}\n\n.tree-outline-disclosure > li {\n    border-top: 1px solid #f0f0f0;\n}\n\n.tree-outline-disclosure > li:first-of-type {\n    border-top: none;\n}\n\n.tree-outline-disclosure {\n    padding-left: 0 !important;\n    padding-right: 3px;\n}\n\n.tree-outline-disclosure li.parent::before {\n    top: 0 !important;\n}\n\n.tree-outline-disclosure .name {\n    color: rgb(136, 19, 145);\n}\n\n.tree-outline-disclosure .object-value-node {\n    overflow: hidden;\n    text-overflow: ellipsis;\n}\n\n.event-listener-details {\n    display: flex;\n}\n\n.event-listener-tree-subtitle {\n    float: right;\n    margin-left: 5px;\n    flex-shrink: 0;\n}\n\n.event-listener-button {\n    padding: 0 3px;\n    background-color: #f2f2f2;\n    border-radius: 3px;\n    border: 1px solid #c3c3c3;\n    margin-left: 10px;\n    display: block;\n    cursor: pointer;\n    opacity: 0.8;\n    flex-shrink: 0;\n}\n\n.event-listener-button:hover {\n    background-color: #e0e0e0;\n    opacity: 1;\n}\n\n.tree-outline-disclosure li:hover .event-listener-button {\n    display: inline;\n}\n\n/*# sourceURL=event_listeners/eventListenersView.css */";