export const PortTypes={In:Symbol('In'),Out:Symbol('Out'),Param:Symbol('Param'),};self.WebAudio=self.WebAudio||{};WebAudio=WebAudio||{};WebAudio.GraphVisualizer=WebAudio.GraphVisualizer||{};WebAudio.GraphVisualizer.Size;WebAudio.GraphVisualizer.Point;WebAudio.GraphVisualizer.NodeLayout;WebAudio.GraphVisualizer.PortTypes=PortTypes;WebAudio.GraphVisualizer.Port;WebAudio.GraphVisualizer.NodeCreationData;WebAudio.GraphVisualizer.ParamCreationData;WebAudio.GraphVisualizer.NodesConnectionData;WebAudio.GraphVisualizer.NodesDisconnectionData;WebAudio.GraphVisualizer.NodesDisconnectionDataWithDestination;WebAudio.GraphVisualizer.NodeParamConnectionData;WebAudio.GraphVisualizer.NodeParamDisconnectionData;