import'./platform/platform.js';import'./text_utils/text_utils.js';import'./common/common.js';import'./heap_snapshot_model/heap_snapshot_model.js';import'./heap_snapshot_worker/heap_snapshot_worker.js';if(!self.Root||!self.Root.Runtime){self.importScripts('Runtime.js');}
Root.Runtime.startWorker('heap_snapshot_worker');