import'./NativeFunctions.js';import'./JavaScriptMetadata.js';import*as JavaScriptMetadata from'./JavaScriptMetadata.js';import*as NativeFunctions from'./NativeFunctions.js';export{JavaScriptMetadata,NativeFunctions,};