import'./InspectorBackend.js';import'./NodeURL.js';import'../InspectorBackendCommands.js';import*as InspectorBackend from'./InspectorBackend.js';import*as NodeURL from'./NodeURL.js';export{InspectorBackend,NodeURL,};